#include "global.hh"
#include "Cfile.hh"
#include "Cdata_reader.hh"

int Cdata_reader::read_in_one_line(Cfile* file)
{
  /* read in any one line */
  if(file->read_line(m_buf, sizeof(m_buf)) == END_OF_FILE){
    return END_OF_FILE;
  }

  /* check comment */
  if(check_comment() == COMMENT){
    return COMMENT;
  }

  /* inport data from string */
  data_t** data =  scan_data_from_str(m_buf, m_number_of_word);
  if(data == 0){
    printf("scan_data_from_str error\n");
    return ERROR;
  }
  delete [] m_data;
  m_data = new data_t[m_number_of_word];
  for(int i = 0; i < m_number_of_word; ++i){ 
    m_data[i] = *data[i];
    delete data[i];
  }
  delete [] data;

  return OK;
}

int Cdata_reader::get_number_of_word()
{
  return m_number_of_word;
}

int Cdata_reader::get_data(data_t& data, int i)
{
  if(i < 0 || m_number_of_word <= i ){
    data = 0;
    printf("cannot access to m_data[%d]\n", i);
    return ERROR;
  }
  data = m_data[i];
  return OK;
}

int Cdata_reader::check_comment()
{
  char comment;
  int i(0);

  while(1){
    comment = m_buf[i];
    if(comment == ' '){
      ++i;
      continue;
    }else if(comment == '\n' || comment == 'c' || comment == '%' ||
	     comment == '!'  || comment == '#' || comment == '/' ){
      return COMMENT;
    }else{
      return NOT_COMMENT;
    }
  }
}

data_t** Cdata_reader::scan_data_from_str(char* str, int &data_num)
{
  int data_length(1);
  data_t** data = new data_t*[data_length];

  data_num = 0;
  int start(0);
  while(1){
    /* 可変数メモリ確保 */
    if(data_length < data_num + 2){
      data_length = 2 * (data_num + 2);
      data_t** temp_data = new data_t*[data_length];
      for(int i = 0 ; i < data_num; ++i){ 
	temp_data[i] = data[i];
      }
      delete [] data;
      data = temp_data;
      temp_data = 0;
    }
    data[data_num] = new data_t;
    data[data_num + 1] = new data_t;
    int read_flag = read_one_data(str, start, data[data_num], data[data_num + 1]);
    if(read_flag == END_OF_LINE){
      delete data[data_num];
      delete data[data_num + 1];
      break;
    }else if(read_flag == READ_ONE_DATA){
      data_num += 2;
      if((data_num + 2) > (BLOCK_SIZE/sizeof(data_t) - (DATA_NUM + 1))){
	printf("to many word in one line\n");
	return 0;
      }
    }else{
      printf("never to come here, read flag error");
      return 0;
    }
  }
  
  return data;
}

int Cdata_reader::read_one_data(char* str, int &start, data_t* data1, data_t* data2)
{
  int i(start - 1), j(0);
  int float_flag(INT);
  int read_one_data(0);
  int data_buf_length(1);
  char* data_buf = new char[data_buf_length];

  while(1){
    ++i;
    /* 可変数メモリ確保 */
    if(data_buf_length < j + 1){
      data_buf_length = 2 * (j + 1);
      char* temp_data_buf = new char[data_buf_length];
      for(int k = 0 ; k < j; ++k){ 
	temp_data_buf[k] = data_buf[k];
      }
      delete [] data_buf;
      data_buf = temp_data_buf;
      temp_data_buf = 0;
    }
    if(str[i] == '\n'){
      if(read_one_data == 1){
	break;
      }else{
	return END_OF_LINE;
      }
    }else if(str[i] == ' ' ){
      if(read_one_data == 1){
	break;
      }else{
	continue;
      }
    }else if(str[i] == '.'){
      float_flag = FLOAT;
      data_buf[j] = str[i];
      ++j;
      continue;
    }else if(check_num(str[i]) == NUMBER){
      read_one_data = 1;
      data_buf[j] = str[i];
      ++j;
    }else{
      printf("string is mixed in data line\n");  
      return ERROR;
    }
  }
  data_buf[j] = '\0';
  start = i;
  convert_str_to_data(data_buf, float_flag, data1, data2);
  delete [] data_buf;
  return READ_ONE_DATA;
}

int Cdata_reader::check_num(char c)
{
  switch (c) {
  case '0':
    break;
  case '1':
    break;
  case '2':
    break;
  case '3':
    break;
  case '4':
    break;
  case '5':
    break;
  case '6':
    break;
  case '7':
    break;
  case '8':
    break;
  case '9':
    break;
  case '-':
    break;
  case '+':
    break;
  case 'E':
    break;
  case 'e':
    break;
  default:
    return NOT_NUMBER;
  } 
  
  return NUMBER;
}

int Cdata_reader::convert_str_to_data(char* data_buf, int float_flag,
			data_t* data1, data_t* data2 ) 
{
  d_data_t buf;

  if(float_flag == INT){
    if(sscanf(data_buf, "%d", &buf) != 1){
      printf("data is maybe broken at here\n");
      return ERROR;
    }
  }else if(float_flag == FLOAT){
    if(sscanf(data_buf, "%f", &buf) != 1){
      printf("data is maybe broken at here\n");
      return ERROR;
    }
  }else{
    printf("never to come here, float_flag if not INT of FLOAT\n");
    return ERROR;
  }

  char* pt = (char*)&buf;
  memcpy(data1, pt, sizeof(data_t));
  memcpy(data2, pt + sizeof(data_t), sizeof(data_t));

  return OK;
}
