#include "global.hh"
#include "Cfile.hh"
#include "Cconverter.hh"
#include "rdfconverter.hh"

int main(int argc, char *argv[])
{
  /* check data_t size */
  if(sizeof(data_t) != (size_t)2){
    printf("\nsizeof(data_t) must be 2byte!!\n");
    printf("please configure source(edit #define data_t)\n\n");
    return ERROR;
  }

  /* check and inport arguments */
  std::string option, binary_filename, txt_filename;
  if(check_arguments(argc, argv, option, binary_filename, txt_filename) != OK){
    return ERROR;
  };

  /* set files and convert */
  Cfile* file_in;
  Cfile* file_out;
  Cconverter* converter;

  file_in = new Cfile();
  file_out = new Cfile();

  if(option == "-b2rb" || option == "-b2t"){  
    if(file_in->fopen_rb(binary_filename) != OK){
      return ERROR;
    }
    if(file_out->fopen_w(txt_filename) != OK){
      return ERROR;
    }  
    converter = new Cconverter();
    converter->set_file(file_in, "in");
    converter->set_file(file_out, "out");

    if(option == "-b2rb"){
      if(converter->convert_binary_to_readable_binary() == ERROR){
	printf("convertion did not finish normaly\n");
	return ERROR;
      }
    }else if(option == "-b2t"){
      if(converter->convert_binary_to_txt() == ERROR){
	printf("convertion did not finish normaly\n");
	return ERROR;
      }
    }else{
      printf("never to come here\n");
    }
  }else if(option == "-t2b"){  
    if(file_in->fopen_r(binary_filename) != OK){
      return ERROR;
    }
    if(file_out->fopen_wb(txt_filename) != OK){
      return ERROR;
    }  
    converter = new Cconverter();
    converter->set_file(file_in, "in");
    converter->set_file(file_out, "out");

    if(option == "-t2b"){
      if(converter->convert_txt_to_binary() == ERROR){
	printf("convertion did not finish normaly\n");
	return ERROR;
      }
    }else{
      printf("\n***never to come here***\n");
    }
  }

  delete converter;
  delete file_out;
  delete file_in;

  printf("\n conversion is done!!\n\n");
  
  return OK;
}

int check_arguments(int argc, char *argv[], std::string &option,
		std::string &binary_filename, std::string &txt_filename)
{
  if(argc != 4){
    printf("\n%s%s%s%s\n",
	   "you need to input option, input-filename,and output-filename.\n",
	   "for example : rdfconverter -b2rb 1001.rdf 1001.txt\n\n",
	   "available option is\n",
	   options);
    return ERROR;
  }else{
    if(argv[1][0] != '-'){
      printf("second argument is option\n%s\n", options);
      return ERROR;
    }else{
      option = argv[1];
      if(option != "-b2rb" && option != "-b2t" && option != "-t2b"){
	printf("available option is\n%s\n", options);
	return ERROR;
      }else{
	binary_filename = argv[2];
	txt_filename = argv[3];
      }
    }
  }
  return OK;
}
